/****************************************************************************
** Meta object code from reading C++ file 'openinghours.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kopeninghours-26.04.2/src/lib/openinghours.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'openinghours.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KOpeningHours12OpeningHoursE_t {};
} // unnamed namespace

template <> constexpr inline auto KOpeningHours::OpeningHours::qt_create_metaobjectdata<qt_meta_tag_ZN13KOpeningHours12OpeningHoursE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOpeningHours::OpeningHours",
        "setLocation",
        "",
        "latitude",
        "longitude",
        "interval",
        "KOpeningHours::Interval",
        "dt",
        "nextInterval",
        "error",
        "Error",
        "normalizedExpression",
        "region",
        "timeZone",
        "Modes",
        "Mode",
        "IntervalMode",
        "PointInTimeMode",
        "Null",
        "NoError",
        "SyntaxError",
        "MissingRegion",
        "MissingLocation",
        "IncompatibleMode",
        "UnsupportedFeature",
        "EvaluationError"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'setLocation'
        QtMocHelpers::MethodData<void(float, float)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 3 }, { QMetaType::Float, 4 },
        }}),
        // Method 'interval'
        QtMocHelpers::MethodData<KOpeningHours::Interval(const QDateTime &) const>(5, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::QDateTime, 7 },
        }}),
        // Method 'nextInterval'
        QtMocHelpers::MethodData<KOpeningHours::Interval(const KOpeningHours::Interval &) const>(8, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { 0x80000000 | 6, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'error'
        QtMocHelpers::PropertyData<enum Error>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'normalizedExpression'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'latitude'
        QtMocHelpers::PropertyData<float>(3, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'longitude'
        QtMocHelpers::PropertyData<float>(4, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'region'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'timeZone'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'Modes'
        QtMocHelpers::EnumData<Modes>(14, 15, QMC::EnumIsFlag).add({
            {   16, Mode::IntervalMode },
            {   17, Mode::PointInTimeMode },
        }),
        // enum 'Error'
        QtMocHelpers::EnumData<enum Error>(10, 10, QMC::EnumFlags{}).add({
            {   18, Error::Null },
            {   19, Error::NoError },
            {   20, Error::SyntaxError },
            {   21, Error::MissingRegion },
            {   22, Error::MissingLocation },
            {   23, Error::IncompatibleMode },
            {   24, Error::UnsupportedFeature },
            {   25, Error::EvaluationError },
        }),
    };
    return QtMocHelpers::metaObjectData<OpeningHours, qt_meta_tag_ZN13KOpeningHours12OpeningHoursE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KOpeningHours::OpeningHours::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOpeningHours12OpeningHoursE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOpeningHours12OpeningHoursE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KOpeningHours12OpeningHoursE_t>.metaTypes,
    nullptr
} };

void KOpeningHours::OpeningHours::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<OpeningHours *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setLocation((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 1: { KOpeningHours::Interval _r = _t->interval((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KOpeningHours::Interval*>(_a[0]) = std::move(_r); }  break;
        case 2: { KOpeningHours::Interval _r = _t->nextInterval((*reinterpret_cast<std::add_pointer_t<KOpeningHours::Interval>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KOpeningHours::Interval*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum Error*>(_v) = _t->error(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->normalizedExpressionString(); break;
        case 2: *reinterpret_cast<float*>(_v) = _t->latitude(); break;
        case 3: *reinterpret_cast<float*>(_v) = _t->longitude(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->region(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->timeZoneId(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setLatitude(*reinterpret_cast<float*>(_v)); break;
        case 3: _t->setLongitude(*reinterpret_cast<float*>(_v)); break;
        case 4: _t->setRegion(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setTimeZoneId(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
