/****************************************************************************
** Meta object code from reading C++ file 'types.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kopeninghours-26.04.2/src/qml/types.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'types.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15IntervalForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto IntervalForeign::qt_create_metaobjectdata<qt_meta_tag_ZN15IntervalForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "IntervalForeign",
        "QML.Foreign",
        "KOpeningHours::Interval",
        "QML.Element",
        "interval"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<IntervalForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject IntervalForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15IntervalForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15IntervalForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15IntervalForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN13IntervalEnumsE_t {};
} // unnamed namespace

template <> constexpr inline auto IntervalEnums::qt_create_metaobjectdata<qt_meta_tag_ZN13IntervalEnumsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "IntervalEnums"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IntervalEnums, qt_meta_tag_ZN13IntervalEnumsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject IntervalEnums::staticMetaObject = { {
    QtPrivate::MetaObjectForType<KOpeningHours::Interval>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13IntervalEnumsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13IntervalEnumsE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13IntervalEnumsE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN20InvervalModelForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto InvervalModelForeign::qt_create_metaobjectdata<qt_meta_tag_ZN20InvervalModelForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "InvervalModelForeign",
        "QML.Foreign",
        "KOpeningHours::IntervalModel",
        "QML.Element",
        "IntervalModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<InvervalModelForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject InvervalModelForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20InvervalModelForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20InvervalModelForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20InvervalModelForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN19OpeningHoursForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto OpeningHoursForeign::qt_create_metaobjectdata<qt_meta_tag_ZN19OpeningHoursForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OpeningHoursForeign",
        "QML.Foreign",
        "KOpeningHours::OpeningHours",
        "QML.Element",
        "openingHours"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<OpeningHoursForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OpeningHoursForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19OpeningHoursForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19OpeningHoursForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19OpeningHoursForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN17OpeningHoursEnumsE_t {};
} // unnamed namespace

template <> constexpr inline auto OpeningHoursEnums::qt_create_metaobjectdata<qt_meta_tag_ZN17OpeningHoursEnumsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OpeningHoursEnums"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OpeningHoursEnums, qt_meta_tag_ZN17OpeningHoursEnumsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OpeningHoursEnums::staticMetaObject = { {
    QtPrivate::MetaObjectForType<KOpeningHours::OpeningHours>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17OpeningHoursEnumsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17OpeningHoursEnumsE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17OpeningHoursEnumsE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN20IntervalEnumsForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto IntervalEnumsForeign::qt_create_metaobjectdata<qt_meta_tag_ZN20IntervalEnumsForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "IntervalEnumsForeign",
        "QML.Element",
        "Interval",
        "QML.Foreign",
        "IntervalEnums",
        "QML.ForeignIsNamespace",
        "true"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN20IntervalEnumsForeignE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN20IntervalEnumsForeignE =
    IntervalEnumsForeign::qt_create_metaobjectdata<qt_meta_tag_ZN20IntervalEnumsForeignE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN20IntervalEnumsForeignE =
    qt_staticMetaObjectContent_ZN20IntervalEnumsForeignE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN20IntervalEnumsForeignE =
    qt_staticMetaObjectContent_ZN20IntervalEnumsForeignE.relocatingData;

Q_CONSTINIT const QMetaObject IntervalEnumsForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN20IntervalEnumsForeignE.stringdata,
    qt_staticMetaObjectStaticContent_ZN20IntervalEnumsForeignE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN20IntervalEnumsForeignE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN24OpeningHoursEnumsForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto OpeningHoursEnumsForeign::qt_create_metaobjectdata<qt_meta_tag_ZN24OpeningHoursEnumsForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OpeningHoursEnumsForeign",
        "QML.Element",
        "OpeningHours",
        "QML.Foreign",
        "OpeningHoursEnums",
        "QML.ForeignIsNamespace",
        "true"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN24OpeningHoursEnumsForeignE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN24OpeningHoursEnumsForeignE =
    OpeningHoursEnumsForeign::qt_create_metaobjectdata<qt_meta_tag_ZN24OpeningHoursEnumsForeignE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN24OpeningHoursEnumsForeignE =
    qt_staticMetaObjectContent_ZN24OpeningHoursEnumsForeignE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN24OpeningHoursEnumsForeignE =
    qt_staticMetaObjectContent_ZN24OpeningHoursEnumsForeignE.relocatingData;

Q_CONSTINIT const QMetaObject OpeningHoursEnumsForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN24OpeningHoursEnumsForeignE.stringdata,
    qt_staticMetaObjectStaticContent_ZN24OpeningHoursEnumsForeignE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN24OpeningHoursEnumsForeignE.metaTypes,
    nullptr
} };

QT_WARNING_POP
