/****************************************************************************
** Meta object code from reading C++ file 'interval.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kopeninghours-26.04.2/src/lib/interval.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'interval.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KOpeningHours8IntervalE_t {};
} // unnamed namespace

template <> constexpr inline auto KOpeningHours::Interval::qt_create_metaobjectdata<qt_meta_tag_ZN13KOpeningHours8IntervalE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOpeningHours::Interval",
        "state",
        "State",
        "begin",
        "hasOpenBegin",
        "end",
        "hasOpenEnd",
        "hasOpenEndTime",
        "comment",
        "estimatedEnd",
        "dstOffset",
        "Invalid",
        "Open",
        "Closed",
        "Unknown"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'state'
        QtMocHelpers::PropertyData<enum State>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'begin'
        QtMocHelpers::PropertyData<QDateTime>(3, QMetaType::QDateTime, QMC::DefaultPropertyFlags),
        // property 'hasOpenBegin'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'end'
        QtMocHelpers::PropertyData<QDateTime>(5, QMetaType::QDateTime, QMC::DefaultPropertyFlags),
        // property 'hasOpenEnd'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'hasOpenEndTime'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'comment'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'estimatedEnd'
        QtMocHelpers::PropertyData<QDateTime>(9, QMetaType::QDateTime, QMC::DefaultPropertyFlags),
        // property 'dstOffset'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(2, 2, QMC::EnumFlags{}).add({
            {   11, State::Invalid },
            {   12, State::Open },
            {   13, State::Closed },
            {   14, State::Unknown },
        }),
    };
    return QtMocHelpers::metaObjectData<Interval, qt_meta_tag_ZN13KOpeningHours8IntervalE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KOpeningHours::Interval::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOpeningHours8IntervalE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOpeningHours8IntervalE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KOpeningHours8IntervalE_t>.metaTypes,
    nullptr
} };

void KOpeningHours::Interval::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Interval *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum State*>(_v) = _t->state(); break;
        case 1: *reinterpret_cast<QDateTime*>(_v) = _t->begin(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->hasOpenBegin(); break;
        case 3: *reinterpret_cast<QDateTime*>(_v) = _t->end(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->hasOpenEnd(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->hasOpenEndTime(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->comment(); break;
        case 7: *reinterpret_cast<QDateTime*>(_v) = _t->estimatedEnd(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->dstOffset(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
