/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<types.h>)
#  include <types.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_kopeninghours()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    QMetaType::fromType<IntervalEnums>().id();
    qmlRegisterNamespaceAndRevisions(&IntervalEnums::staticMetaObject, "org.kde.kopeninghours", 1, nullptr, &IntervalEnumsForeign::staticMetaObject, nullptr);
    qmlRegisterTypesAndRevisions<IntervalForeign>("org.kde.kopeninghours", 1);
    qmlRegisterEnum<KOpeningHours::Interval::State>("KOpeningHours::Interval::State");
    qmlRegisterTypesAndRevisions<InvervalModelForeign>("org.kde.kopeninghours", 1);
    QMetaType::fromType<OpeningHoursEnums>().id();
    qmlRegisterNamespaceAndRevisions(&OpeningHoursEnums::staticMetaObject, "org.kde.kopeninghours", 1, nullptr, &OpeningHoursEnumsForeign::staticMetaObject, nullptr);
    qmlRegisterTypesAndRevisions<OpeningHoursForeign>("org.kde.kopeninghours", 1);
    qmlRegisterEnum<KOpeningHours::OpeningHours::Modes>("KOpeningHours::OpeningHours::Modes");
    qmlRegisterEnum<KOpeningHours::OpeningHours::Mode>("KOpeningHours::OpeningHours::Mode");
    qmlRegisterEnum<KOpeningHours::OpeningHours::Error>("KOpeningHours::OpeningHours::Error");
    QMetaType::fromType<QAbstractItemModel *>().id();
    qmlRegisterEnum<QAbstractItemModel::LayoutChangeHint>("QAbstractItemModel::LayoutChangeHint");
    qmlRegisterEnum<QAbstractItemModel::CheckIndexOption>("QAbstractItemModel::CheckIndexOption");
    QMetaType::fromType<QAbstractListModel *>().id();
    QT_WARNING_POP
    qmlRegisterModule("org.kde.kopeninghours", 1, 0);
}

static const QQmlModuleRegistration orgkdekopeninghoursRegistration("org.kde.kopeninghours", qml_register_types_org_kde_kopeninghours);
