/*
    SPDX-FileCopyrightText: 2020 David Faure <faure@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef PYTHON_QT_WRAPPERS_H
#define PYTHON_QT_WRAPPERS_H

void register_qt_wrappers();

#endif // PYTHON_QT_WRAPPERS_H
