// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#ifndef ECM_QLOGGINGCATEGORY_KOPENINGHOURS_LOG_LOGGING_H
#define ECM_QLOGGINGCATEGORY_KOPENINGHOURS_LOG_LOGGING_H

#include <QLoggingCategory>
 namespace KOpeningHours {
Q_DECLARE_LOGGING_CATEGORY(Log)
} 

#endif
